@@ -15,6 +15,9 @@ plot results from source coordinates to the GeoAxes' target projection.
 import collections
 import contextlib
 import functools
+import json
+import os
+from pathlib import Path
 import warnings
 import weakref
 
@@ -23,50 +26,43 @@ import matplotlib.artist
 import matplotlib.axes
 import matplotlib.contour
 from matplotlib.image import imread
-import matplotlib.transforms as mtransforms
 import matplotlib.patches as mpatches
 import matplotlib.path as mpath
 import matplotlib.spines as mspines
+import matplotlib.transforms as mtransforms
 import numpy as np
 import numpy.ma as ma
+import packaging
 import shapely.geometry as sgeom
 
 from cartopy import config
 import cartopy.crs as ccrs
 import cartopy.feature
 import cartopy.mpl.contour
-import cartopy.mpl.geocollection
 import cartopy.mpl.feature_artist as feature_artist
+import cartopy.mpl.geocollection
 import cartopy.mpl.patch as cpatch
 from cartopy.mpl.slippy_image_artist import SlippyImageArtist
 
 
-assert mpl.__version__ >= '3.1', \
-    'Cartopy is only supported with Matplotlib 3.1 or greater.'
+_MPL_VERSION = packaging.version.parse(mpl.__version__)
+assert _MPL_VERSION.release >= (3, 4), \
+    'Cartopy is only supported with Matplotlib 3.4 or greater.'
 
+# A nested mapping from path, source CRS, and target projection to the
+# resulting transformed paths:
+#     {path: {(source_crs, target_projection): list_of_paths}}
+# Provides a significant performance boost for contours which, at
+# matplotlib 1.2.0 called transform_path_non_affine twice unnecessarily.
 _PATH_TRANSFORM_CACHE = weakref.WeakKeyDictionary()
-"""
-A nested mapping from path, source CRS, and target projection to the
-resulting transformed paths::
-
-    {path: {(source_crs, target_projection): list_of_paths}}
-
-Provides a significant performance boost for contours which, at
-matplotlib 1.2.0 called transform_path_non_affine twice unnecessarily.
-
-"""
 
+# A dictionary of pre-loaded images for large background images, kept as a
+# dictionary so that large images are loaded only once.
 _BACKG_IMG_CACHE = {}
-"""
-A dictionary of pre-loaded images for large background images, kept as a
-dictionary so that large images are loaded only once.
-"""
 
+# A dictionary of background images in the directory specified by the
+# CARTOPY_USER_BACKGROUNDS environment variable.
 _USER_BG_IMGS = {}
-"""
-A dictionary of background images in the directory specified by the
-CARTOPY_USER_BACKGROUNDS environment variable.
-"""
 
 
 # XXX call this InterCRSTransform
@@ -252,7 +248,7 @@ class _ViewClippedPathPatch(mpatches.PathPatch):
     def _adjust_location(self):
         if self.stale:
             self.set_path(self._original_path.clip_to_bbox(self.axes.viewLim))
-            # Some places in matplotlib's tranform stack cache the actual
+            # Some places in matplotlib's transform stack cache the actual
             # path so we trigger an update by invalidating the transform.
             self._trans_wrap.invalidate()
 
@@ -267,7 +263,6 @@ class GeoSpine(mspines.Spine):
         self._original_path = mpath.Path(np.empty((0, 2)))
         kwargs.setdefault('clip_on', False)
         super().__init__(axes, 'geo', self._original_path, **kwargs)
-        self.set_capstyle('butt')
 
     def set_boundary(self, path, transform):
         self._original_path = path
@@ -277,6 +272,7 @@ class GeoSpine(mspines.Spine):
     def _adjust_location(self):
         if self.stale:
             self._path = self._original_path.clip_to_bbox(self.axes.viewLim)
+            self._path = mpath.Path(self._path.vertices, closed=True)
 
     def get_window_extent(self, renderer=None):
         # make sure the location is updated so that transforms etc are
@@ -292,6 +288,7 @@ class GeoSpine(mspines.Spine):
         return ret
 
     def set_position(self, position):
+        """GeoSpine does not support changing its position."""
         raise NotImplementedError(
             'GeoSpine does not support changing its position.')
 
@@ -438,7 +435,7 @@ class GeoAxes(matplotlib.axes.Axes):
             Currently an image "factory" is just an object with
             an ``image_for_domain`` method. Examples of image factories
             are :class:`cartopy.io.img_nest.NestedImageCollection` and
-            :class:`cartopy.io.image_tiles.GoogleTiles`.
+            :class:`cartopy.io.img_tiles.GoogleTiles`.
 
         """
         if hasattr(factory, 'image_for_domain'):
@@ -634,7 +631,8 @@ class GeoAxes(matplotlib.axes.Axes):
         resolution : str or :class:`cartopy.feature.Scaler`, optional
             A named resolution to use from the Natural Earth
             dataset. Currently can be one of "auto" (default), "110m", "50m",
-            and "10m", or a Scaler object.
+            and "10m", or a Scaler object.  If "auto" is selected, the
+            resolution is defined by `~cartopy.feature.auto_scaler`.
 
         """
         kwargs['edgecolor'] = color
@@ -1010,11 +1008,9 @@ class GeoAxes(matplotlib.axes.Axes):
 
         """
         if name == 'ne_shaded':
-            import os
             source_proj = ccrs.PlateCarree()
-            fname = os.path.join(config["repo_data_dir"],
-                                 'raster', 'natural_earth',
-                                 '50-natural-earth-1-downsampled.png')
+            fname = (config["repo_data_dir"] / 'raster' / 'natural_earth'
+                     / '50-natural-earth-1-downsampled.png')
 
             return self.imshow(imread(fname), origin='upper',
                                transform=source_proj,
@@ -1064,11 +1060,9 @@ class GeoAxes(matplotlib.axes.Axes):
         # read in the user's background image directory:
         if len(_USER_BG_IMGS) == 0:
             self.read_user_background_images()
-        import os
-        bgdir = os.getenv('CARTOPY_USER_BACKGROUNDS')
-        if bgdir is None:
-            bgdir = os.path.join(config["repo_data_dir"],
-                                 'raster', 'natural_earth')
+        bgdir = Path(os.getenv(
+            'CARTOPY_USER_BACKGROUNDS',
+            config["repo_data_dir"] / 'raster' / 'natural_earth'))
         # now get the filename we want to use:
         try:
             fname = _USER_BG_IMGS[name][resolution]
@@ -1078,7 +1072,7 @@ class GeoAxes(matplotlib.axes.Axes):
                 f'present in the user background image metadata in directory '
                 f'{bgdir!r}')
         # Now obtain the image data from file or cache:
-        fpath = os.path.join(bgdir, fname)
+        fpath = bgdir / fname
         if cache:
             if fname in _BACKG_IMG_CACHE:
                 img = _BACKG_IMG_CACHE[fname]
@@ -1164,14 +1158,10 @@ class GeoAxes(matplotlib.axes.Axes):
         lib/cartopy/data/raster/natural_earth/images.json
 
         """
-        import os
-        import json
-
-        bgdir = os.getenv('CARTOPY_USER_BACKGROUNDS')
-        if bgdir is None:
-            bgdir = os.path.join(config["repo_data_dir"],
-                                 'raster', 'natural_earth')
-        json_file = os.path.join(bgdir, 'images.json')
+        bgdir = Path(os.getenv(
+            'CARTOPY_USER_BACKGROUNDS',
+            config["repo_data_dir"] / 'raster' / 'natural_earth'))
+        json_file = bgdir / 'images.json'
 
         with open(json_file) as js_obj:
             dict_in = json.load(js_obj)
@@ -1196,8 +1186,8 @@ class GeoAxes(matplotlib.axes.Axes):
                         # the required_info items are not resolutions:
                         if resln not in required_info:
                             img_it_r = _USER_BG_IMGS[img_type][resln]
-                            test_file = os.path.join(bgdir, img_it_r)
-                            if not os.path.isfile(test_file):
+                            test_file = bgdir / img_it_r
+                            if not test_file.is_file():
                                 raise ValueError(
                                     f'File "{test_file}" not found')
 
@@ -1614,12 +1604,15 @@ class GeoAxes(matplotlib.axes.Axes):
         result = super().contour(*args, **kwargs)
 
         # We need to compute the dataLim correctly for contours.
-        bboxes = [col.get_datalim(self.transData)
-                  for col in result.collections
-                  if col.get_paths()]
-        if bboxes:
-            extent = mtransforms.Bbox.union(bboxes)
-            self.dataLim.update_from_data_xy(extent.get_points(), ignore=False)
+        if _MPL_VERSION.release[:2] < (3, 8):
+            bboxes = [col.get_datalim(self.transData)
+                      for col in result.collections
+                      if col.get_paths()]
+            if bboxes:
+                extent = mtransforms.Bbox.union(bboxes)
+                self.update_datalim(extent.get_points())
+        else:
+            self.update_datalim(result.get_datalim(self.transData))
 
         self.autoscale_view()
 
@@ -1662,12 +1655,15 @@ class GeoAxes(matplotlib.axes.Axes):
         result = super().contourf(*args, **kwargs)
 
         # We need to compute the dataLim correctly for contours.
-        bboxes = [col.get_datalim(self.transData)
-                  for col in result.collections
-                  if col.get_paths()]
-        if bboxes:
-            extent = mtransforms.Bbox.union(bboxes)
-            self.dataLim.update_from_data_xy(extent.get_points(), ignore=False)
+        if _MPL_VERSION.release[:2] < (3, 8):
+            bboxes = [col.get_datalim(self.transData)
+                      for col in result.collections
+                      if col.get_paths()]
+            if bboxes:
+                extent = mtransforms.Bbox.union(bboxes)
+                self.update_datalim(extent.get_points())
+        else:
+            self.update_datalim(result.get_datalim(self.transData))
 
         self.autoscale_view()
 
@@ -1797,9 +1793,7 @@ class GeoAxes(matplotlib.axes.Axes):
         Handle the interpolation when a wrap could be involved with
         the data coordinates before passing on to Matplotlib.
         """
-        # The shading keyword argument was added in MPL 3.3, so keep
-        # this default updating until we only support MPL>=3.3
-        default_shading = mpl.rcParams.get('pcolor.shading', 'auto')
+        default_shading = mpl.rcParams.get('pcolor.shading')
         if not (kwargs.get('shading', default_shading) in
                 ('nearest', 'auto') and len(args) == 3 and
                 getattr(kwargs.get('transform'), '_wrappable', False)):
@@ -1810,7 +1804,7 @@ class GeoAxes(matplotlib.axes.Axes):
         kwargs['shading'] = 'flat'
         X = np.asanyarray(args[0])
         Y = np.asanyarray(args[1])
-        nrows, ncols = np.asanyarray(args[2]).shape
+        nrows, ncols = np.asanyarray(args[2]).shape[:2]
         Nx = X.shape[-1]
         Ny = Y.shape[0]
         if X.ndim != 2 or X.shape[0] == 1:
@@ -1857,12 +1851,13 @@ class GeoAxes(matplotlib.axes.Axes):
         Ny, Nx, _ = coords.shape
         if kwargs.get('shading') == 'gouraud':
             # Gouraud shading has the same shape for coords and data
-            data_shape = Ny, Nx
+            data_shape = Ny, Nx, -1
         else:
-            data_shape = Ny - 1, Nx - 1
+            data_shape = Ny - 1, Nx - 1, -1
         # data array
         C = collection.get_array().reshape(data_shape)
-
+        if C.shape[-1] == 1:
+            C = C.squeeze(axis=-1)
         transformed_pts = self.projection.transform_points(
             t, coords[..., 0], coords[..., 1])
 
@@ -1935,13 +1930,12 @@ class GeoAxes(matplotlib.axes.Axes):
                           "map it must be fully transparent.",
                           stacklevel=3)
 
-        # The original data mask (regardless of wrapped cells)
-        C_mask = getattr(C, 'mask', None)
+        # Get hold of masked versions of the array to be passed to set_array
+        # methods of QuadMesh and PolyQuadMesh
+        pcolormesh_data, pcolor_data, pcolor_mask = \
+            cartopy.mpl.geocollection._split_wrapped_mesh_data(C, mask)
 
-        # create the masked array to be used with this pcolormesh
-        full_mask = mask if C_mask is None else mask | C_mask
-        pcolormesh_data = np.ma.array(C, mask=full_mask)
-        collection.set_array(pcolormesh_data.ravel())
+        collection.set_array(pcolormesh_data)
 
         # plot with slightly lower zorder to avoid odd issue
         # where the main plot is obscured
@@ -1957,25 +1951,32 @@ class GeoAxes(matplotlib.axes.Axes):
         # `pcolor` only draws polygons where the data is not
         # masked, so this will only draw a limited subset of
         # polygons that were actually wrapped.
-        # We will add the original data mask in later to
-        # make sure that set_array can work in future
-        # calls on the proper sized array inputs.
-        # NOTE: we don't use C.data here because C.data could
-        #       contain nan's which would be masked in the
-        #       pcolor routines, which we don't want. We will
-        #       fill in the proper data later with set_array()
-        #       calls.
-        pcolor_data = np.ma.array(np.zeros(C.shape),
-                                  mask=~mask)
-        pcolor_col = self.pcolor(coords[..., 0], coords[..., 1],
-                                 pcolor_data, zorder=zorder,
-                                 **kwargs)
-        # Now add back in the masked data if there was any
-        full_mask = ~mask if C_mask is None else ~mask | C_mask
-        pcolor_data = np.ma.array(C, mask=full_mask)
-        # The pcolor_col is now possibly shorter than the
-        # actual collection, so grab the masked cells
-        pcolor_col.set_array(pcolor_data[mask].ravel())
+
+        if _MPL_VERSION.release[:2] < (3, 8):
+            # We will add the original data mask in later to
+            # make sure that set_array can work in future
+            # calls on the proper sized array inputs.
+            # NOTE: we don't use C.data here because C.data could
+            #       contain nan's which would be masked in the
+            #       pcolor routines, which we don't want. We will
+            #       fill in the proper data later with set_array()
+            #       calls.
+            pcolor_zeros = np.ma.array(np.zeros(C.shape), mask=pcolor_mask)
+            pcolor_col = self.pcolor(coords[..., 0], coords[..., 1],
+                                     pcolor_zeros, zorder=zorder,
+                                     **kwargs)
+
+            # The pcolor_col is now possibly shorter than the
+            # actual collection, so grab the masked cells
+            pcolor_col.set_array(pcolor_data[mask].ravel())
+        else:
+            pcolor_col = self.pcolor(coords[..., 0], coords[..., 1],
+                                     pcolor_data, zorder=zorder,
+                                     **kwargs)
+            # Currently pcolor_col.get_array() will return a compressed array
+            # and warn unless we explicitly set the 2D array.  This should be
+            # unnecessary with future matplotlib versions.
+            pcolor_col.set_array(pcolor_data)
 
         pcolor_col.set_cmap(cmap)
         pcolor_col.set_norm(norm)
@@ -1986,7 +1987,7 @@ class GeoAxes(matplotlib.axes.Axes):
         # put the pcolor_col and mask on the pcolormesh
         # collection so that users can do things post
         # this method
-        collection._wrapped_mask = mask.ravel()
+        collection._wrapped_mask = mask
         collection._wrapped_collection_fix = pcolor_col
 
         return collection
@@ -2005,10 +2006,6 @@ class GeoAxes(matplotlib.axes.Axes):
         # Add in an argument checker to handle Matplotlib's potential
         # interpolation when coordinate wraps are involved
         args, kwargs = self._wrap_args(*args, **kwargs)
-        if matplotlib.__version__ < "3.3":
-            # MPL 3.3 introduced the shading option, and it isn't
-            # handled before that for pcolor calls.
-            kwargs.pop('shading', None)
         result = super().pcolor(*args, **kwargs)
 
         # Update the datalim for this pcolor.
