@@ -5,28 +5,30 @@
 # licensing details.
 
 import matplotlib.pyplot as plt
-from matplotlib.testing.decorators import cleanup
 import pytest
 
 import cartopy.crs as ccrs
-from cartopy.tests.mpl import ImageTesting, MPL_VERSION
+from cartopy.tests.mpl import MPL_VERSION
 
 
 @pytest.mark.natural_earth
-@ImageTesting(["igh_land"],
-              tolerance=(3.6 if MPL_VERSION.release[:2] < (3, 5)
-                         or ccrs.PROJ_VERSION < (9, 0, 1) else 0.5))
+@pytest.mark.mpl_image_compare(filename="igh_land.png",
+                               tolerance=(3.6
+                                          if MPL_VERSION.release[:2] < (3, 5)
+                                          else 0.5))
 def test_igh_land():
     crs = ccrs.InterruptedGoodeHomolosine(emphasis="land")
     ax = plt.axes(projection=crs)
     ax.coastlines()
     ax.gridlines()
+    return ax.figure
 
 
 @pytest.mark.natural_earth
-@ImageTesting(["igh_ocean"],
-              tolerance=(4.5 if MPL_VERSION.release[:2] < (3, 5)
-                         or ccrs.PROJ_VERSION < (9, 0, 1) else 0.5))
+@pytest.mark.mpl_image_compare(filename="igh_ocean.png",
+                               tolerance=(4.5
+                                          if MPL_VERSION.release[:2] < (3, 5)
+                                          else 0.5))
 def test_igh_ocean():
     crs = ccrs.InterruptedGoodeHomolosine(
         central_longitude=-160, emphasis="ocean"
@@ -34,10 +36,11 @@ def test_igh_ocean():
     ax = plt.axes(projection=crs)
     ax.coastlines()
     ax.gridlines()
+    return ax.figure
 
 
 @pytest.mark.natural_earth
-@ImageTesting(['lambert_conformal_south'])
+@pytest.mark.mpl_image_compare(filename='lambert_conformal_south.png')
 def test_lambert_south():
     # Reference image: https://www.icsm.gov.au/mapping/map_projections.html
     crs = ccrs.LambertConformal(central_longitude=140, cutoff=65,
@@ -45,10 +48,10 @@ def test_lambert_south():
     ax = plt.axes(projection=crs)
     ax.coastlines()
     ax.gridlines()
+    return ax.figure
 
 
 @pytest.mark.natural_earth
-@cleanup
 def test_repr_html():
     pc = ccrs.PlateCarree()
     html = pc._repr_html_()
