@@ -6,12 +6,12 @@
 
 import re
 
-import numpy as np
+import matplotlib.colors as mcolors
 import matplotlib.pyplot as plt
+import numpy as np
 import pytest
 
 import cartopy.crs as ccrs
-
 from cartopy.tests.mpl import MPL_VERSION
 
 
@@ -159,6 +159,7 @@ def test_simple_global():
 @pytest.mark.filterwarnings("ignore:Unable to determine extent")
 @pytest.mark.natural_earth
 @pytest.mark.parametrize('proj', [
+    ccrs.Aitoff,
     ccrs.EckertI,
     ccrs.EckertII,
     ccrs.EckertIII,
@@ -167,6 +168,7 @@ def test_simple_global():
     ccrs.EckertVI,
     ccrs.EqualEarth,
     ccrs.Gnomonic,
+    ccrs.Hammer,
     pytest.param((ccrs.InterruptedGoodeHomolosine, dict(emphasis='land')),
                  id='InterruptedGoodeHomolosine'),
     ccrs.LambertCylindrical,
@@ -186,6 +188,13 @@ def test_simple_global():
     ccrs.SouthPolarStereo,
     pytest.param((ccrs.TransverseMercator, dict(approx=True)),
                  id='TransverseMercator'),
+    pytest.param(
+        (ccrs.ObliqueMercator, dict(azimuth=0.)), id='ObliqueMercator_default'
+    ),
+    pytest.param(
+        (ccrs.ObliqueMercator, dict(azimuth=90., central_latitude=-22)),
+        id='ObliqueMercator_rotated',
+    ),
 ])
 @pytest.mark.mpl_image_compare(
     tolerance=0.97 if MPL_VERSION.release[:2] < (3, 5) else 0.5,
@@ -232,10 +241,46 @@ def test_cursor_values():
                      r.encode('ascii', 'ignore'))
 
 
+SKIP_PRE_MPL38 = pytest.mark.skipif(
+    MPL_VERSION.release[:2] < (3, 8), reason='mpl < 3.8')
+PARAMETRIZE_PCOLORMESH_WRAP = pytest.mark.parametrize(
+    'mesh_data_kind',
+    [
+        'standard',
+        pytest.param('rgb', marks=SKIP_PRE_MPL38),
+        pytest.param('rgba', marks=SKIP_PRE_MPL38),
+    ],
+    ids=['standard', 'rgb', 'rgba'],
+)
+
+
+def _to_rgb(data, mesh_data_kind):
+    """
+    Helper function to convert array to RGB(A) where required
+    """
+    if mesh_data_kind in ('rgb', 'rgba'):
+        cmap = plt.get_cmap()
+        norm = mcolors.Normalize()
+        new_data = cmap(norm(data))
+        if mesh_data_kind == 'rgb':
+            new_data = new_data[..., 0:3]
+            if np.ma.is_masked(data):
+                # Use data's mask as an alpha channel
+                mask = np.ma.getmaskarray(data)
+                mask = np.broadcast_to(
+                    mask[..., np.newaxis], new_data.shape).copy()
+                new_data = np.ma.array(new_data, mask=mask)
+
+        return new_data
+
+    return data
+
+
+@PARAMETRIZE_PCOLORMESH_WRAP
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_global_wrap1.png',
                                tolerance=1.27)
-def test_pcolormesh_global_with_wrap1():
+def test_pcolormesh_global_with_wrap1(mesh_data_kind):
     # make up some realistic data with bounds (such as data from the UM)
     nx, ny = 36, 18
     xbnds = np.linspace(0, 360, nx, endpoint=True)
@@ -246,6 +291,8 @@ def test_pcolormesh_global_with_wrap1():
     data = data[:-1, :-1]
     fig = plt.figure()
 
+    data = _to_rgb(data, mesh_data_kind)
+
     ax = fig.add_subplot(2, 1, 1, projection=ccrs.PlateCarree())
     ax.pcolormesh(xbnds, ybnds, data, transform=ccrs.PlateCarree(), snap=False)
     ax.coastlines()
@@ -259,7 +306,8 @@ def test_pcolormesh_global_with_wrap1():
     return fig
 
 
-def test_pcolormesh_get_array_with_mask():
+@PARAMETRIZE_PCOLORMESH_WRAP
+def test_pcolormesh_get_array_with_mask(mesh_data_kind):
     # make up some realistic data with bounds (such as data from the UM)
     nx, ny = 36, 18
     xbnds = np.linspace(0, 360, nx, endpoint=True)
@@ -267,7 +315,10 @@ def test_pcolormesh_get_array_with_mask():
 
     x, y = np.meshgrid(xbnds, ybnds)
     data = np.exp(np.sin(np.deg2rad(x)) + np.cos(np.deg2rad(y)))
+    data[5, :] = np.nan  # Check that missing data is handled - GH#2208
     data = data[:-1, :-1]
+    data = _to_rgb(data, mesh_data_kind)
+
     fig = plt.figure()
 
     ax = fig.add_subplot(2, 1, 1, projection=ccrs.PlateCarree())
@@ -275,8 +326,11 @@ def test_pcolormesh_get_array_with_mask():
     assert c._wrapped_collection_fix is not None, \
         'No pcolormesh wrapping was done when it should have been.'
 
-    assert np.array_equal(data.ravel(), c.get_array()), \
-        'Data supplied does not match data retrieved in wrapped case'
+    result = c.get_array()
+    np.testing.assert_array_equal(np.ma.getmask(result), np.isnan(data))
+    np.testing.assert_array_equal(
+        data, result,
+        err_msg='Data supplied does not match data retrieved in wrapped case')
 
     ax.coastlines()
     ax.set_global()  # make sure everything is visible
@@ -288,7 +342,9 @@ def test_pcolormesh_get_array_with_mask():
 
     x, y = np.meshgrid(xbnds, ybnds)
     data = np.exp(np.sin(np.deg2rad(x)) + np.cos(np.deg2rad(y)))
+    data[5, :] = np.nan
     data2 = data[:-1, :-1]
+    data2 = _to_rgb(data2, mesh_data_kind)
 
     ax = fig.add_subplot(2, 1, 2, projection=ccrs.PlateCarree())
     c = ax.pcolormesh(xbnds, ybnds, data2, transform=ccrs.PlateCarree())
@@ -298,14 +354,23 @@ def test_pcolormesh_get_array_with_mask():
     assert getattr(c, "_wrapped_collection_fix", None) is None, \
         'pcolormesh wrapping was done when it should not have been.'
 
-    assert np.array_equal(data2.ravel(), c.get_array()), \
-        'Data supplied does not match data retrieved in unwrapped case'
+    result = c.get_array()
+
+    expected = data2
+    if MPL_VERSION.release[:2] < (3, 8):
+        expected = expected.ravel()
+
+    np.testing.assert_array_equal(np.ma.getmask(result), np.isnan(expected))
+    np.testing.assert_array_equal(
+        expected, result,
+        'Data supplied does not match data retrieved in unwrapped case')
 
 
+@PARAMETRIZE_PCOLORMESH_WRAP
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_global_wrap2.png',
                                tolerance=1.87)
-def test_pcolormesh_global_with_wrap2():
+def test_pcolormesh_global_with_wrap2(mesh_data_kind):
     # make up some realistic data with bounds (such as data from the UM)
     nx, ny = 36, 18
     xbnds, xstep = np.linspace(0, 360, nx - 1, retstep=True, endpoint=True)
@@ -320,6 +385,8 @@ def test_pcolormesh_global_with_wrap2():
     data = data[:-1, :-1]
     fig = plt.figure()
 
+    data = _to_rgb(data, mesh_data_kind)
+
     ax = fig.add_subplot(2, 1, 1, projection=ccrs.PlateCarree())
     ax.pcolormesh(xbnds, ybnds, data, transform=ccrs.PlateCarree(), snap=False)
     ax.coastlines()
@@ -333,10 +400,11 @@ def test_pcolormesh_global_with_wrap2():
     return fig
 
 
+@PARAMETRIZE_PCOLORMESH_WRAP
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_global_wrap3.png',
                                tolerance=1.42)
-def test_pcolormesh_global_with_wrap3():
+def test_pcolormesh_global_with_wrap3(mesh_data_kind):
     nx, ny = 33, 17
     xbnds = np.linspace(-1.875, 358.125, nx, endpoint=True)
     ybnds = np.linspace(91.25, -91.25, ny, endpoint=True)
@@ -354,6 +422,8 @@ def test_pcolormesh_global_with_wrap3():
     data = np.ma.masked_greater(data, 2.6)
     fig = plt.figure()
 
+    data = _to_rgb(data, mesh_data_kind)
+
     ax = fig.add_subplot(3, 1, 1, projection=ccrs.PlateCarree(-45))
     c = ax.pcolormesh(xbnds, ybnds, data, transform=ccrs.PlateCarree(),
                       snap=False)
@@ -376,10 +446,11 @@ def test_pcolormesh_global_with_wrap3():
     return fig
 
 
+@PARAMETRIZE_PCOLORMESH_WRAP
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_global_wrap3.png',
                                tolerance=1.42)
-def test_pcolormesh_set_array_with_mask():
+def test_pcolormesh_set_array_with_mask(mesh_data_kind):
     """Testing that set_array works with masked arrays properly."""
     nx, ny = 33, 17
     xbnds = np.linspace(-1.875, 358.125, nx, endpoint=True)
@@ -402,10 +473,15 @@ def test_pcolormesh_set_array_with_mask():
     bad_data_mask = np.ma.array(bad_data, mask=~data.mask)
     fig = plt.figure()
 
+    data = _to_rgb(data, mesh_data_kind)
+    bad_data = _to_rgb(bad_data, mesh_data_kind)
+    bad_data_mask = _to_rgb(bad_data_mask, mesh_data_kind)
+
     ax = fig.add_subplot(3, 1, 1, projection=ccrs.PlateCarree(-45))
     c = ax.pcolormesh(xbnds, ybnds, bad_data,
                       norm=norm, transform=ccrs.PlateCarree(), snap=False)
-    c.set_array(data.ravel())
+
+    c.set_array(data)
     assert c._wrapped_collection_fix is not None, \
         'No pcolormesh wrapping was done when it should have been.'
 
@@ -415,7 +491,10 @@ def test_pcolormesh_set_array_with_mask():
     ax = fig.add_subplot(3, 1, 2, projection=ccrs.PlateCarree(-1.87499952))
     c2 = ax.pcolormesh(xbnds, ybnds, bad_data_mask,
                        norm=norm, transform=ccrs.PlateCarree(), snap=False)
-    c2.set_array(data.ravel())
+    if mesh_data_kind == 'standard':
+        c2.set_array(data.ravel())
+    else:
+        c2.set_array(data)
     ax.coastlines()
     ax.set_global()  # make sure everything is visible
 
@@ -427,6 +506,37 @@ def test_pcolormesh_set_array_with_mask():
     return fig
 
 
+def test_pcolormesh_set_array_nowrap():
+    # Roundtrip check that set_array works with the correct shaped arrays
+    nx, ny = 36, 18
+    xbnds = np.linspace(-60, 60, nx, endpoint=True)
+    ybnds = np.linspace(-80, 80, ny, endpoint=True)
+    xbnds, ybnds = np.meshgrid(xbnds, ybnds)
+
+    rng = np.random.default_rng()
+    data = rng.random((ny - 1, nx - 1))
+
+    ax = plt.figure().add_subplot(projection=ccrs.PlateCarree())
+    mesh = ax.pcolormesh(xbnds, ybnds, data)
+    assert not hasattr(mesh, '_wrapped_collection_fix')
+
+    expected = data
+    if MPL_VERSION.release[:2] < (3, 8):
+        expected = expected.ravel()
+    np.testing.assert_array_equal(mesh.get_array(), expected)
+
+    # For backwards compatibility, check we can set a 1D array
+    data = rng.random((nx - 1) * (ny - 1))
+    mesh.set_array(data)
+    np.testing.assert_array_equal(
+        mesh.get_array(), data.reshape(ny - 1, nx - 1))
+
+    # Check that we can set a 2D array even if previous was flat
+    data = rng.random((ny - 1, nx - 1))
+    mesh.set_array(data)
+    np.testing.assert_array_equal(mesh.get_array(), data)
+
+
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_global_wrap3.png',
                                tolerance=1.42)
@@ -571,6 +681,13 @@ def test_pcolormesh_nan_wrap():
     ax = plt.axes(projection=ccrs.PlateCarree())
     mesh = ax.pcolormesh(xs, ys, data)
     pcolor = getattr(mesh, "_wrapped_collection_fix")
+    if MPL_VERSION.release[:2] < (3, 8):
+        assert len(pcolor.get_paths()) == 2
+    else:
+        assert not pcolor.get_paths()
+
+    # Check that we can populate the pcolor with some data.
+    mesh.set_array(np.ones((2, 2)))
     assert len(pcolor.get_paths()) == 2
 
 
@@ -605,14 +722,18 @@ def test_pcolormesh_mercator_wrap():
     return ax.figure
 
 
+@PARAMETRIZE_PCOLORMESH_WRAP
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare(filename='pcolormesh_mercator_wrap.png')
-def test_pcolormesh_wrap_set_array():
+def test_pcolormesh_wrap_set_array(mesh_data_kind):
     x = np.linspace(0, 360, 73)
     y = np.linspace(-87.5, 87.5, 36)
     X, Y = np.meshgrid(*[np.deg2rad(c) for c in (x, y)])
     Z = np.cos(Y) + 0.375 * np.sin(2. * X)
     Z = Z[:-1, :-1]
+
+    Z = _to_rgb(Z, mesh_data_kind)
+
     ax = plt.axes(projection=ccrs.Mercator())
     norm = plt.Normalize(np.min(Z), np.max(Z))
     ax.coastlines()
@@ -620,7 +741,7 @@ def test_pcolormesh_wrap_set_array():
     coll = ax.pcolormesh(x, y, np.ones(Z.shape), norm=norm,
                          transform=ccrs.PlateCarree(), snap=False)
     # Now update the plot with the set_array method
-    coll.set_array(Z.ravel())
+    coll.set_array(Z)
     return ax.figure
 
 
@@ -861,7 +982,7 @@ def test_streamplot():
 @pytest.mark.natural_earth
 @pytest.mark.mpl_image_compare()
 def test_annotate():
-    """ test a variety of annotate options on mulitple projections
+    """ test a variety of annotate options on multiple projections
 
     Annotate defaults to coords passed as if they're in map projection space.
     `transform` or `xycoords` & `textcoords` control the marker and text offset
@@ -896,7 +1017,7 @@ def test_annotate():
 
     # set up coordinates in map projection space
     map_coords = map_projection.transform_point(-175, -35, platecarree)
-    # Dont specifiy any args, default xycoords='data', transform=map projection
+    # Don't specify any args, default xycoords='data', transform=map projection
     ax.annotate('default crs', map_coords, size=5)
 
     # data in map projection using default transform, with
