@@ -14,8 +14,7 @@ from cartopy.io.ogc_clients import _OWSLIB_AVAILABLE
 @pytest.mark.filterwarnings("ignore:TileMatrixLimits")
 @pytest.mark.network
 @pytest.mark.skipif(not _OWSLIB_AVAILABLE, reason='OWSLib is unavailable.')
-@pytest.mark.xfail(raises=KeyError, reason='OWSLib WMTS support is broken.')
-@pytest.mark.mpl_image_compare(filename='wmts.png', tolerance=0)
+@pytest.mark.mpl_image_compare(filename='wmts.png', tolerance=0.03)
 def test_wmts():
     ax = plt.axes(projection=ccrs.PlateCarree())
     url = 'https://map1c.vis.earthdata.nasa.gov/wmts-geo/wmts.cgi'
