@@ -9,10 +9,12 @@ from io import BytesIO
 import os
 from pathlib import Path
 import pickle
+import warnings
 
 import numpy as np
-from numpy.testing import assert_almost_equal, assert_array_equal
+from numpy.testing import assert_almost_equal
 from numpy.testing import assert_array_almost_equal as assert_arr_almost_eq
+from numpy.testing import assert_array_equal
 import pyproj
 import pytest
 import shapely.geometry as sgeom
@@ -89,9 +91,17 @@ class TestCRS:
     def test_epsg(self):
         uk = ccrs.epsg(27700)
         assert uk.epsg_code == 27700
-        assert_almost_equal(uk.x_limits, (-104009.357, 688806.007), decimal=3)
-        assert_almost_equal(uk.y_limits, (-8908.37, 1256558.45), decimal=2)
-        assert_almost_equal(uk.threshold, 7928.15, decimal=2)
+        expected_x = (-104009.357, 688806.007)
+        expected_y = (-8908.37, 1256558.45)
+        expected_threshold = 7928.15
+        if pyproj.__proj_version__ >= '9.2.0':
+            expected_x = (-104728.764, 688806.007)
+            expected_y = (-8908.36, 1256616.32)
+            expected_threshold = 7935.34
+        assert_almost_equal(uk.x_limits,
+                            expected_x, decimal=3)
+        assert_almost_equal(uk.y_limits, expected_y, decimal=2)
+        assert_almost_equal(uk.threshold, expected_threshold, decimal=2)
         self._check_osgb(uk)
 
     def test_epsg_compound_crs(self):
@@ -342,3 +352,13 @@ def test_projection__from_string():
 
 def test_crs__from_pyproj_crs():
     assert ccrs.CRS(pyproj.CRS("EPSG:4326")) == "EPSG:4326"
+
+
+def test_transform_point_no_warning():
+    # Make sure we aren't warning on single-point numpy arrays
+    # see https://github.com/SciTools/cartopy/pull/2194
+    p = ccrs.PlateCarree()
+    p2 = ccrs.Mercator()
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+        p2.transform_point(1, 2, p)
