@@ -1,32 +1,18 @@
-# (C) British Crown Copyright 2011 - 2019, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
-
-from __future__ import (absolute_import, division, print_function)
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 
 import numpy as np
 import pytest
 import shapely.geometry as sgeom
 import shapely.wkt
 
-
 import cartopy.crs as ccrs
 
 
-class TestBoundary(object):
+class TestBoundary:
     def test_no_polygon_boundary_reversal(self):
         # Check that polygons preserve their clockwise or counter-clockwise
         # ordering when they are attached to the boundary.
@@ -35,7 +21,7 @@ class TestBoundary(object):
         polygon = sgeom.Polygon([(-10, 30), (10, 60), (10, 50)])
         projection = ccrs.Robinson(170.5)
         multi_polygon = projection.project_geometry(polygon)
-        for polygon in multi_polygon:
+        for polygon in multi_polygon.geoms:
             assert polygon.is_valid
 
     def test_polygon_boundary_attachment(self):
@@ -68,10 +54,10 @@ class TestBoundary(object):
         for coords, expected_polys in polys:
             polygon = sgeom.Polygon(coords)
             multi_polygon = projection.project_geometry(polygon)
-            assert len(multi_polygon) == expected_polys
+            assert len(multi_polygon.geoms) == expected_polys
 
 
-class TestMisc(object):
+class TestMisc:
     def test_misc(self):
         projection = ccrs.TransverseMercator(central_longitude=-90,
                                              approx=False)
@@ -86,8 +72,8 @@ class TestMisc(object):
             (-179.9173693847652942, -16.5017831356493616),
         ])
         multi_polygon = projection.project_geometry(polygon)
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].exterior.coords) == 4
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].exterior.coords) == 4
 
     def test_former_infloop_case(self):
         # test a polygon which used to get stuck in an infinite loop
@@ -188,8 +174,8 @@ class TestMisc(object):
                                  (200000, -1000)])
         multi_polygon = projection.project_geometry(polygon,
                                                     ccrs.OSGB(approx=True))
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].exterior.coords) == 4
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].exterior.coords) == 4
 
     def test_self_intersecting_1(self):
         # Geometry comes from a matplotlib contourf (see #537)
@@ -210,7 +196,7 @@ class TestMisc(object):
         # approximately 13262233761329.
         area = projected.area
         assert 2.2e9 < area < 2.3e9, \
-            'Got area {}, expecting ~2.2e9'.format(area)
+            f'Got area {area}, expecting ~2.2e9'
 
     def test_self_intersecting_2(self):
         # Geometry comes from a matplotlib contourf (see #509)
@@ -238,7 +224,7 @@ class TestMisc(object):
         # Before fixing, this geometry used to fill the whole disk. Approx
         # 1.2e14.
         assert 81330 < area < 81340, \
-            'Got area {}, expecting ~81336'.format(area)
+            f'Got area {area}, expecting ~81336'
 
     def test_same_points_on_boundary_1(self):
         source = ccrs.PlateCarree()
@@ -261,14 +247,49 @@ class TestMisc(object):
 
         assert abs(1200 - projected.area) < 1e-5
 
+    def test_attach_short_loop(self):
+        # Geometry comes from a matplotlib contourf.
+        mstring = shapely.wkt.loads(
+            'MULTILINESTRING ('
+            '(-179.9999982118607 71.87500000000001,'
+            '-179.0625 71.87500000000001,'
+            '-179.9999982118607 71.87500000000001))')
+        multi_line_strings = [mstring]
 
-class TestQuality(object):
+        src = ccrs.PlateCarree()
+        polygons = src._attach_lines_to_boundary(multi_line_strings, True)
+        # Before fixing, this would contain a geometry which would
+        # cause a segmentation fault.
+        assert polygons == []
+
+    def test_project_degenerate_poly(self):
+        # Tests for the same bug as test_attach_short_loop.
+        # This test calls only the public API, but will cause a
+        # segmentation fault when it fails.
+        # Geometry comes from a matplotlib contourf.
+        polygon = shapely.wkt.loads(
+            'POLYGON (('
+            '178.9687499944748 70.625, '
+            '179.0625 71.875, '
+            '180.9375 71.875, '
+            '179.0625 71.875, '
+            '177.1875 71.875, '
+            '178.9687499944748 70.625))')
+
+        source = ccrs.PlateCarree()
+        target = ccrs.PlateCarree()
+        # Before fixing, this would cause a segmentation fault.
+        polygons = target.project_geometry(polygon, source)
+        assert type(polygons) == sgeom.MultiPolygon
+
+
+class TestQuality:
     def setup_class(self):
         projection = ccrs.RotatedPole(pole_longitude=177.5,
                                       pole_latitude=37.5)
         polygon = sgeom.Polygon([
             (177.5, -57.38460319),
-            (180.0, -57.445077),
+            (180.1, -57.445077),
             (175.0, -57.19913331),
         ])
         self.multi_polygon = projection.project_geometry(polygon)
@@ -277,14 +298,14 @@ class TestQuality(object):
 
     def test_split(self):
         # Start simple ... there should be two projected polygons.
-        assert len(self.multi_polygon) == 2
+        assert len(self.multi_polygon.geoms) == 2
 
     def test_repeats(self):
         # Make sure we don't have repeated points at the boundary, because
         # they mess up the linear extrapolation to the boundary.
 
         # Make sure there aren't any repeated points.
-        xy = np.array(self.multi_polygon[0].exterior.coords)
+        xy = np.array(self.multi_polygon.geoms[0].exterior.coords)
         same = (xy[1:] == xy[:-1]).all(axis=1)
         assert not any(same), 'Repeated points in projected geometry.'
 
@@ -294,7 +315,7 @@ class TestQuality(object):
         # from the boundary.
 
         # Identify all the contiguous sets of non-boundary points.
-        xy = np.array(self.multi_polygon[0].exterior.coords)
+        xy = np.array(self.multi_polygon.geoms[0].exterior.coords)
         boundary = np.logical_or(xy[:, 1] == 90, xy[:, 1] == -90)
         regions = (boundary[1:] != boundary[:-1]).cumsum()
         regions = np.insert(regions, 0, 0)
@@ -310,7 +331,7 @@ class TestQuality(object):
             assert abs(num_incr - num_decr) < 3, 'Too much asymmetry.'
 
 
-class PolygonTests(object):
+class PolygonTests:
     def _assert_bounds(self, bounds, x1, y1, x2, y2, delta=1):
         assert abs(bounds[0] - x1) < delta
         assert abs(bounds[1] - y1) < delta
@@ -320,15 +341,15 @@ class PolygonTests(object):
 
 class TestWrap(PolygonTests):
     # Test that Plate Carree projection "does the right thing"(tm) with
-    # source data tha extends outside the [-180, 180] range.
+    # source data that extends outside the [-180, 180] range.
     def test_plate_carree_no_wrap(self):
         proj = ccrs.PlateCarree()
         poly = sgeom.box(0, 0, 10, 10)
         multi_polygon = proj.project_geometry(poly, proj)
         # Check the structure
-        assert len(multi_polygon) == 1
+        assert len(multi_polygon.geoms) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, 0, 0, 10, 10)
 
     def test_plate_carree_partial_wrap(self):
@@ -336,9 +357,9 @@ class TestWrap(PolygonTests):
         poly = sgeom.box(170, 0, 190, 10)
         multi_polygon = proj.project_geometry(poly, proj)
         # Check the structure
-        assert len(multi_polygon) == 2
+        assert len(multi_polygon.geoms) == 2
         # Check the rough shape
-        poly1, poly2 = multi_polygon
+        poly1, poly2 = multi_polygon.geoms
         # The order of these polygons is not guaranteed, so figure out
         # which is appropriate
         if 170.0 not in poly1.bounds:
@@ -351,9 +372,9 @@ class TestWrap(PolygonTests):
         poly = sgeom.box(200, 0, 220, 10)
         multi_polygon = proj.project_geometry(poly, proj)
         # Check the structure
-        assert len(multi_polygon) == 1
+        assert len(multi_polygon.geoms) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, -160, 0, -140, 10)
 
 
@@ -369,10 +390,10 @@ class TestHoles(PolygonTests):
                              [ring(-20, -20, 20, 20, False)])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].interiors) == 1
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].interiors) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, -40, -47, 40, 47)
         self._assert_bounds(polygon.interiors[0].bounds, -20, -21, 20, 21)
 
@@ -382,9 +403,9 @@ class TestHoles(PolygonTests):
                              [ring(-20, -20, 20, 20, False)])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 2
+        assert len(multi_polygon.geoms) == 2
 
-        poly1, poly2 = multi_polygon
+        poly1, poly2 = multi_polygon.geoms
         # The order of these polygons is not guaranteed, so figure out
         # which is appropriate
         if not len(poly1.interiors) == 1:
@@ -403,13 +424,13 @@ class TestHoles(PolygonTests):
                              [ring(-20, -20, 20, 20, False)])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 2
-        assert len(multi_polygon[0].interiors) == 0
-        assert len(multi_polygon[1].interiors) == 0
+        assert len(multi_polygon.geoms) == 2
+        assert len(multi_polygon.geoms[0].interiors) == 0
+        assert len(multi_polygon.geoms[1].interiors) == 0
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, 140, -47, 180, 47)
-        polygon = multi_polygon[1]
+        polygon = multi_polygon.geoms[1]
         self._assert_bounds(polygon.bounds, -180, -47, -140, 47)
 
     def test_inverted_poly_simple_hole(self):
@@ -418,10 +439,10 @@ class TestHoles(PolygonTests):
                              [[(0, -30), (90, -30), (180, -30), (270, -30)]])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].interiors) == 1
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].interiors) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, -2.4e7, -2.4e7, 2.4e7, 2.4e7, 1e6)
         self._assert_bounds(polygon.interiors[0].bounds,
                             - 1.2e7, -1.2e7, 1.2e7, 1.2e7, 1e6)
@@ -433,10 +454,10 @@ class TestHoles(PolygonTests):
                                (45, -60), (135, -60)]])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].interiors) == 1
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].interiors) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, -5.0e7, -5.0e7, 5.0e7, 5.0e7, 1e6)
         self._assert_bounds(polygon.interiors[0].bounds,
                             - 1.2e7, -1.2e7, 1.2e7, 1.2e7, 1e6)
@@ -449,10 +470,10 @@ class TestHoles(PolygonTests):
                                (45, -75), (135, -75)]])
         multi_polygon = proj.project_geometry(poly)
         # Check the structure
-        assert len(multi_polygon) == 1
-        assert len(multi_polygon[0].interiors) == 1
+        assert len(multi_polygon.geoms) == 1
+        assert len(multi_polygon.geoms[0].interiors) == 1
         # Check the rough shape
-        polygon = multi_polygon[0]
+        polygon = multi_polygon.geoms[0]
         self._assert_bounds(polygon.bounds, -5.0e7, -5.0e7, 5.0e7, 5.0e7, 1e6)
         self._assert_bounds(polygon.interiors[0].bounds,
                             - 1.2e7, -1.2e7, 1.2e7, 1.2e7, 1e6)
@@ -467,4 +488,4 @@ class TestHoles(PolygonTests):
         target = ccrs.PlateCarree()
         source = ccrs.Geodetic()
 
-        assert len(list(target.project_geometry(poly, source))) == 1
+        assert len(list(target.project_geometry(poly, source).geoms)) == 1
